#include "impr.h"
#include "helpers.h"

/**
 * impr_manhattan_distance
 * @dest: destination image
 * @src: source image
 * @w: image width
 * @h: image height
 *
 * Fill matrix with manhattan distances to closest on-pixel (1).
 * Manhattan distance is defined as: D = |x2-x1| + |y2-y1|
 * Image edges are ignored for speed.
 * dest == src is ok.
 */
void impr_manhattan_distance(u8 *dest, b8 *src, int w, int h)
{
	int i, j;

	for (i=0; i<w; i++)
		dest[i] = src[i]-1; /* underflow to U8_MAX */

	/* calculate it for right and down */
	for (j=1; j<h; j++) {
		int pos = j*w;
		dest[pos] = src[pos]-1;
		for (i=1; i<w; i++) {
			pos++;
			dest[pos] = src[pos]-1;
			dest[pos] = min(dest[pos], dest[pos-1]+1);
			dest[pos] = min(dest[pos], dest[pos-w]+1);
		}
	}

	/* and for left and up */
	for (j=h-2; j>=0; j--) {
		int pos = j*w + w-2;
		for (i=w-2; i>=0; i--) {
			dest[pos] = min(dest[pos], dest[pos+1]+1);
			dest[pos] = min(dest[pos], dest[pos+w]+1);
			pos--;
		}
	}
}

/**
 * impr_chessboard_distance
 * @dest: destination image
 * @src: source image
 * @w: image width
 * @h: image height
 *
 * Fill matrix with chessboard distances to closest on-pixel (1).
 * Chessboard distance is defined as: D = max(|x2-x1|, |y2-y1|)
 * Image edges are ignored for speed.
 * dest == src is ok.
 */
void impr_chessboard_distance(u8 *dest, b8 *src, int w, int h)
{
	int i, j;

	for (i=0; i<w; i++)
		dest[i] = src[i]-1; /* underflow to U8_MAX */

	/* calculate it for right and down */
	for (j=1; j<h; j++) {
		int pos = j*w;
		dest[pos] = src[pos]-1;
		for (i=1; i<w-1; i++) {
			u8 tmp;
			pos++;
			tmp = src[pos]-1;
			tmp = min(tmp, dest[pos-1]+1);
			tmp = min(tmp, dest[pos-w]+1);
			tmp = min(tmp, dest[pos-w-1]+1);
			tmp = min(tmp, dest[pos-w+1]+1);
			dest[pos] = tmp;
		}
	}

	/* and for left and up */
	for (j=h-2; j>=0; j--) {
		int pos = j*w + w-2;
		for (i=w-2; i>=1; i--) {
			u8 tmp = dest[pos];
			tmp = min(tmp, dest[pos+1]+1);
			tmp = min(tmp, dest[pos+w]+1);
			tmp = min(tmp, dest[pos+w+1]+1);
			tmp = min(tmp, dest[pos+w-1]+1);
			dest[pos] = tmp;
			pos--;
		}
	}
}
