#include "filter.h"
#include "helpers.h"

/**
 * filter_roberts_thresh
 * @dest: destination binarized image
 * @src: source grey image
 * @w: image width
 * @h: image height
 * @thresh: threshold above (or equal to) which the value is considered one
 *
 * Applies roberts filter to grey image, and binarizes to values >= @thresh.
 * Resulting image is shifted by 0.5 pixel to left and up.
 * @dest == @src is ok.
 */
void filter_roberts_thresh(b8 *dest, u8 *src, int w, int h, int thresh)
{
	int i;
	int pos = 0;
	int next_pos = w;	/* next line */

	while (--h) {
		i = w;
		while (--i) {
			int tmp = absi((int)src[pos] - src[next_pos+1]);
			tmp = max(tmp, absi((int)src[pos+1] - src[next_pos]));

			dest[pos] = tmp >= thresh;

			pos++;
			next_pos++;
		}
		pos++;	/* the remaining pixel */
		next_pos++;
	}
}

/**
 * filter_smooth_2
 * @dest: destination image
 * @src: source image
 * @w: image width
 * @h: image height
 *
 * Apply a 2x2 smoothening filter. p[y][x] = avg(p[y..y+1][x..x+1])
 * @dest == @src is ok.
 */
void filter_smooth_2(u8 *dest, u8 *src, int w, int h)
{
	int i, j;
	int pos = 0;
	int next_pos = w;	/* next line */

	for (j=0; j<h-1; j++) {
		for (i=0; i<w-1; i++) {
			int tmp = src[pos];
			tmp += src[next_pos];
			tmp += src[pos+1];
			tmp += src[next_pos+1];

			dest[pos] = tmp/4;
			pos++;
			next_pos++;
		}
		pos++;	/* the remaining pixel */
		next_pos++;
	}
}
