#include "filter.h"
#include "helpers.h"

/* i don't like these implementations too much */

static inline void _filter_roberts(u8 *dest, u8 *src, int w, int h, int add_to)
{
	int i, j;
	int pos = 0;
	int next_pos = w;	/* next line */

	for (j=0; j<h-1; j++) {
		for (i=0; i<w-1; i++) {
			int tmp = absi((int)src[pos] - src[next_pos+1]);
			tmp += absi((int)src[pos+1] - src[next_pos]);
			/**
			 * use lower-right pixel, because segmentation cuts upper right
			 * border anyway */
			if (add_to)
				dest[next_pos+1] = min(tmp + dest[next_pos+1], U8_MAX);
			else
				dest[next_pos+1] = min(tmp, U8_MAX);

			pos++;
			next_pos++;
		}
		pos++;	/* the remaining pixel */
		next_pos++;
	}
}
__deprecated void filter_roberts(u8 *dest, u8 *src, int w, int h)
{
	_filter_roberts(dest, src, w, h, 0);
}

__deprecated void filter_roberts_add(u8 *dest, u8 *src, int w, int h)
{
	_filter_roberts(dest, src, w, h, 1);
}

static inline void _filter_sobel(u8 *dest, u8 *src, int w, int h, int add_to)
{
	int i, j;
	int prev_pos = 0;
	int pos = w;
	int next_pos = 2*w;

	for (j=1; j<h-1; j++) {
		for (i=1; i<w-1; i++) {
			int tmpx;
			int tmpy;
			tmpx =  (int)src[prev_pos] + 2*src[prev_pos+1] + src[prev_pos+2];
			tmpx -= (int)src[next_pos] + 2*src[next_pos+1] + src[next_pos+2];
			/* ^ this was edge detection along x, this will be along y: */
			tmpy =  (int)src[prev_pos]   + 2*src[pos]   + src[next_pos];
			tmpy -= (int)src[prev_pos+2] + 2*src[pos+2] + src[next_pos+2];

			if (add_to)
				dest[pos+1] = min(dest[pos+1] + absi(tmpx) + absi(tmpy), U8_MAX);
			else
				dest[pos+1] = min(absi(tmpx) + absi(tmpy), U8_MAX);

			prev_pos++;
			pos++;
			next_pos++;
		}
		prev_pos += 2;
		pos += 2;	/* the remaining 2 pixels */
		next_pos += 2;
	}
}

__deprecated void filter_sobel(u8 *dest, u8 *src, int w, int h)
{
	_filter_sobel(dest, src, w, h, 0);
}

__deprecated void filter_sobel_add(u8 *dest, u8 *src, int w, int h)
{
	_filter_sobel(dest, src, w, h, 1);
}

/* adds to dest */
static inline void _filter_laplace(u8 *dest, u8 *src, int w, int h, int add_to)
{
	int i, j;
	int prev_pos = 0;
	int pos = w;
	int next_pos = 2*w;

	for (j=1; j<h-1; j++) {
		for (i=1; i<w-1; i++) {
			int tmp;
			/*tmp =  (int)src[prev_pos] + src[prev_pos+1] + src[prev_pos+2];
			tmp += (int)src[pos]    - 8*src[pos+1]      + src[pos+2];
			tmp += (int)src[next_pos] + src[next_pos+1] + src[next_pos+2];*/
			tmp =                  (int)src[prev_pos+1];
			tmp += (int)src[pos]    - 4*src[pos+1]      + src[pos+2];
			tmp +=                 (int)src[next_pos+1];

			if (add_to)
				dest[pos+1] = min(dest[pos+1] + absi(tmp), U8_MAX);
			else
				dest[pos+1] = min(absi(tmp), U8_MAX);

			prev_pos++;
			pos++;
			next_pos++;
		}
		prev_pos += 2;
		pos += 2;	/* the remaining 2 pixels */
		next_pos += 2;
	}
}

__deprecated void filter_laplace(u8 *dest, u8 *src, int w, int h)
{
	_filter_laplace(dest, src, w, h, 0);
}

__deprecated void filter_laplace_add(u8 *dest, u8 *src, int w, int h)
{
	_filter_laplace(dest, src, w, h, 1);
}

__deprecated void filter_smooth_x(u8 *dest, u8 *src, int w, int h, int size)
{
	int i, j;
	int offset = (size-1)/2;

	for (j=0; j<h-size+1; j++)
		for (i=0; i<w-size+1; i++) {
			int k, l;
			int tmp = 0;

			for (l=0; l<size; l++)
				for (k=0; k<size; k++)
					tmp += src[(j+l)*w + i+k];

			dest[j*w + i] = tmp / (size*size);
		}

	for (j=h-1; j>=offset; j--)
		for (i=w-1; i>=offset; i--)
			dest[j*w + i] = dest[(j-offset)*w + (i-offset)];
}
