#include "impr.h"
#include "queue.h"

/**
 * impr_strip_border
 * @_chan: binary image in which to fill the holes
 * @w: image width
 * @h: image height
 *
 * Sets image borders to 0..
 */
void impr_strip_border(u8_or_b8p _chan, int w, int h)
{
	u8 *chan = _chan._u;
	int i;
	int size = w*h;

	for (i=0; i<w; i++) {
		chan[i] = 0;
		chan[i+size-w] = 0;
	}
	for (i=0; i<size; i+=w) {
		chan[i] = 0;
		chan[i+w-1] = 0;
	}
}

/* helpers, so generic queue (int) can be used */
#define _pack(x, y) (((y)<<16) | (x))
#define _unpack(x, y, el) do { int _tmp = el; x=_tmp&0xffff; y=_tmp>>16; } while (0)

/**
 * impr_flood
 * @chan: image in which to flood an object
 * @w: image width
 * @h: image height
 * @x: x coordinate of flooded object
 * @y: y coordinate of flooded object
 * @newv: pixel value to be flooded with
 *
 * Floods an object in an image.
 */
void impr_flood(u8 *chan, int w, int h, int x, int y, u8 newv)
{
	struct queue *q = queue_new(0xbad);
	u8 *src = chan;
	u8 *dest = chan;
	u8 old = src[w*y+x];

	enqueue(q, _pack(x, y));

	while (!queue_empty(q)) {
		int pos;
		_unpack(x, y, dequeue(q));

		pos = w*y+x;
		dest[pos] = newv;

		if (x > 0 && src[pos-1] == old) {
			dest[pos-1] = newv;
			enqueue(q, _pack(x-1, y));
		}
		if (x < w-1 && src[pos+1] == old) {
			dest[pos+1] = newv;
			enqueue(q, _pack(x+1, y));
		}
		if (y > 0 && src[pos-w] == old) {
			dest[pos-w] = newv;
			enqueue(q, _pack(x, y-1));
		}
		if (y < h-1 && src[pos+w] == old) {
			dest[pos+w] = newv;
			enqueue(q, _pack(x, y+1));
		}
	}
	queue_free(q);
}

/**
 * impr_fill_holes
 * @chan: binary image in which to fill the holes
 * @w: image width
 * @h: image height
 *
 * Fills holes in a binary image.
 * This function strips borders of the image.
 */
void impr_fill_holes(b8 *chan, int w, int h)
{
	u8 *ch = (u8*)chan;
	impr_strip_border(chan, w, h);
	impr_flood(ch, w, h, 0, 0, 2);
	impr_binarize(chan, ch, w*h, != 2);
}
