#ifndef _IMBW_H_
#define _IMBW_H_

#include "impr.h"

void impr_negate(b8 *dest, b8 *src, int size);
void impr_dilaten(b8 *dest, b8 *src, int w, int h, int conn, int times);
void impr_eroden(b8 *dest, b8 *src, int w, int h, int conn, int times);

/**
 * impr_binarize
 * @dest: destination image
 * @src: source image
 * @size: image size
 * @expr: expression to binarize with.
 *
 * Binarize gray image. Ie. @expr can be >5, !=3,...
 * @dest == @src is ok (delta type safety).
 */
#define impr_binarize(dest, src, size, expr)	\
do {						\
	b8 *_dest = dest;			\
	u8 *_src = src;				\
	int _size = size;			\
	int i;					\
	for (i=0; i<_size; i++)			\
		_dest[i] = _src[i] expr;	\
} while (0)


/**
 * impr_dilate
 * @dest: destination image
 * @src: source image
 * @w: image width
 * @h: image height
 *
 * 8 connected dilation.
 * @dest == @src is ok.
 */
#define impr_dilate(dest, src, w, h) impr_dilaten(dest, src, w, h, 8, 1)

/**
 * impr_erode
 * @dest: destination image
 * @src: source image
 * @w: image width
 * @h: image height
 *
 * 8 connected erosion.
 * @dest == @src is ok.
 */
#define impr_erode(dest, src, w, h) impr_eroden(dest, src, w, h, 8, 1)

void impr_close(b8 *dest, b8 *src, int w, int h);
void impr_open(b8 *dest, b8 *src, int w, int h);

void impr_and(b8 *dest, b8 *src1, b8 *src2, int size);
void impr_or(b8 *dest, b8 *src1, b8 *src2, int size);

#endif
