#ifndef _IMPR_H_
#define _IMPR_H_

#include <stdint.h>
#include <assert.h>
/* or provide your own assert to deal with errors */

#ifndef u8
#define u8 uint8_t
#endif
#ifndef u16
#define u16 uint16_t
#endif
#ifndef u32
#define u32 uint32_t
#endif

#ifndef U8_MAX
#define U8_MAX ((u8)-1)
#endif
#ifndef U16_MAX
#define U16_MAX ((u16)-1)
#endif
#ifndef U32_MAX
#define U32_MAX ((u32)-1)
#endif

#ifndef __deprecated
#define __deprecated	__attribute__((deprecated))
#endif
#ifndef __may_alias
#define __may_alias	__attribute__((__may_alias__))
#endif
#ifndef __transparent_union
#define __transparent_union __attribute__((__transparent_union__))
#endif

typedef u8 __may_alias b8;	/* b8 is same as u8, but gcc will warn when
				   one is used as another */
typedef union { u8 _u; b8 _b; } u8_or_b8 __transparent_union;
typedef union { u8 *_u; b8 *_b; } u8_or_b8p __transparent_union;
				/* this type is used when b8 and u8 are ok */

#include "colorspace.h"
#include "filter.h"
#include "imbw.h"
#include "segmentize.h"

void impr_flip_values(u8 *dest, u8 *src, int size);
void impr_add_scalar_wrap(u8 *dest, u8 *src, int size, int value);
void impr_add_scalar(u8 *dest, u8 *src, int size, int value);
void impr_add(u8 *dest, u8 *src1, u8 *src2, int size);
void impr_div_scalar(u8 *dest, u8 *src, int size, int value);
void impr_div(u8 *dest, u8 *src1, u8 *src2, int size);
void impr_bin2grey(u8 *dest, b8 *src, int size);


void impr_downscale(u8 *dest, u8 *src, int w, int h, int factor);
void impr_upscale(u8 *dest, u8 *src, int wd, int hd, int factor);
void impr_downscale2(u8 *dest, u8 *src, int w, int h, int factorx, int factory);
void impr_upscale2(u8 *dest, u8 *src, int w, int h, int factorx, int factory);


void impr_subimage(u8_or_b8p dest, int dw, int dh,
		u8_or_b8p src, int w, int x, int y);
void impr_putimage(u8_or_b8p dest, int w,
		u8_or_b8p src, int dw, int dh, int x, int y);
void impr_strip_border(b8 *chan, int w, int h);
void impr_flood(u8 *chan, int w, int h, int x, int y, u8 newv);
void impr_fill_holes(b8 *chan, int w, int h);

void impr_manhattan_distance(u8 *dest, b8 *src, int w, int h);
void impr_chessboard_distance(u8 *dest, b8 *src, int w, int h);

/* these functions might be weird or whatever. included just for fun */
void impr_subimage16_cmp(b8 *dest, int dw, int dh, u16 *src, int w, int x, int y, u16 value);
__deprecated void impr_movelr(u8_or_b8p dest, u8_or_b8p src, int w, int h, int off);
void impr_blur_dir(u8 *dest, u8 *src, int w, int h, float dir, int len);


#endif /* _IMPR_H_ */
