#include <math.h>
#include "impr.h"
#include "helpers.h"

static void _impr_get_pix_to_blur(float dir, int pix_to_blur[], int len, int w,
		int *xbord, int *ybord)
{
	float xdiff = cosf(dir);
	float ydiff = sinf(dir);
	float x = -xdiff*(len-1)/2.0f;
	float y = -ydiff*(len-1)/2.0f;
	int i;

	*xbord = absi(round(x));
	*ybord = absi(round(y));
	for (i=0; i<len; i++) {
		pix_to_blur[i] = (int)round(x) - w*(int)round(y);
		/* - because y-axis in computer systems is reverse of "normal", cartesian */
		x += xdiff;
		y += ydiff;
	}
}

/* blurs image in direction @dir (angle in radians) */
/* borders are just copied */
void impr_blur_dir(u8 *dest, u8 *src, int w, int h, float dir, int len)
{
	int i, j;
	int pix_to_blur[len];
	int ybord, xbord;

	assert(dest != src);

	_impr_get_pix_to_blur(dir, pix_to_blur, len, w, &xbord, &ybord);

	for (j=ybord; j<h-ybord; j++)
		for (i=xbord; i<w-xbord; i++) {
			int pos = j*w+i;
			int l;
			int tmp = 0;
			for (l=0; l<len; l++)
				tmp += src[pos+pix_to_blur[l]];
			dest[pos] = tmp / len;
		}

	/* copy borders */
	for (j=0; j<h; j++)
		for (i=0; i<w; i++) {
			int pos;

			if ((j>=ybord && j<h-ybord) && i==xbord)
				i = w-xbord;

			pos = j*w+i;

			dest[pos] = src[pos];
		}
}


/* because of 16->8-bit the comparison is a must, this isn't very generic */
void impr_subimage16_cmp(b8 *dest, int dw, int dh, u16 *src, int w, int x, int y, u16 value)
{
	int i, j;
	int spos = 0;
	int ipos = y*w + x;

	for (j=0; j<dh; j++) {
		for (i=0; i<dw; i++)
			dest[spos++] = src[ipos++] == value;

		ipos += w-dw;
	}
}
