#include "impr.h"

/**
 * impr_subimage
 * @_dest: destination (smaller image)
 * @dw: subimage width
 * @dh: subimage height
 * @_src: source (bigger image)
 * @w: image width
 * @x: left border of the subimage
 * @y: upper border of the subimage
 *
 * Extracts a square part of _dest.
 * _dest == _src is ok.
 */
void impr_subimage(u8_or_b8p _dest, int dw, int dh,
		u8_or_b8p _src, int w, int x, int y)
{
	u8 *dest = _dest._u;
	u8 *src = _src._u;
	int i, j;
	int spos = 0;
	int ipos = y*w + x;

	for (j=0; j<dh; j++) {
		for (i=0; i<dw; i++)
			dest[spos++] = src[ipos++];

		ipos += w-dw;
	}
}

/**
 * impr_putimage
 * @_dest: destination (bigger image)
 * @w: image width
 * @_src: source (smaller image)
 * @dw: subimage width
 * @dh: subimage height
 * @x: x coordinate of subimage destination
 * @y: y coordinate of subimage destination
 *
 * Puts _src onto bigger _dest image.
 */
void impr_putimage(u8_or_b8p _dest, int w,
		u8_or_b8p _src, int dw, int dh, int x, int y)
{
	u8 *dest = _dest._u;
	u8 *src = _src._u;
	int i, j;
	int spos = 0;
	int ipos = y*w + x;

	assert(dest != src);

	for (j=0; j<dh; j++) {
		for (i=0; i<dw; i++)
			dest[ipos++] = src[spos++];

		ipos += w-dw;
	}
}
